 /*

The following function can be used to calculate the LRC8 for a string.
Also include are two functions that will convert character representations
of a byte to a number and vise versa.
The code is pure Clipper 5.x. It shouldn't be to hard to convert it to
Clipper '87 flavor if needed.
Use whereever you want.


*------------------------------------------------------------------------

The LRC is the byte that needs to be added to the sum of all the bytes in the
data stream to make it equal 11111111.

Example:
Character       Decimal      Binary

                            7654 3210  <- bits

    H             72        0100 1000
    E             69        0100 0101
    L             76        0100 1100
    L             76        0100 1100
    O             79        0100 1111
                            ---------
                            0100 0010  Total of bits, no carry

                            1011 1101  LRC
                            ---------
Total including LRC must =  1111 1111

*--------------------------------------------------------------------------

Thanks to Terry Carmen for explaining the principle behind this to me.

Anne.

*/


FUNCTION LRC8( cString )
*--------------------------------------------------------------------------
*
* Calculate the LRC8-char for a given string.
*
* cString        The string that the LRC8 has to be calulated for.
* cLRC           The LRC8-Character.
*
* LRC8("HELLO") -> "" (CHR(189))
*
* Num2Byte  Byte2Num
*
*--------------------------------------------------------------------------
LOCAL acBytes := {}, cLRC := "", cTotNC, cChar := CHR(255)
LOCAL nLen, nCount
nLen := LEN(cString)
IF nLen>0
   // Convert string to array of Binary repr. of the ASC() value's //
   FOR i := 1 TO nLen
      AADD(acBytes, Num2Byte( ASC( SUBSTR(cString,i,1) ) ))
   NEXT
   // Do the Binary count of these Bytes, No Carry //
   cTotNC := ""
   FOR i := 1 TO 8                                         // Bit 7 to 0 //
      nCount := 0
      FOR j := 1 TO nLen                                   // For all bytes //
         // Count the times there is a '1' in this bit //
         IF SUBSTR(acBytes[j], i, 1)=="1"
            nCount++
         ENDIF
      NEXT j
      IF nCount%2==0                      // If there's an even times '1' //
         cBit := "0"
      ELSE                                // If there is an Odd times '1' //
         cBit := "1"
      ENDIF
      cTotNC += cBit                      // Build the total for these bytes //
   NEXT i

   // For the LRC-code, invert the string //
   cLRC := ""
   FOR i := 1 TO 8
      cLRC += IF(SUBSTR(cTotNC, i, 1)=="1", "0", "1")      // Invert //
   NEXT i

   // get the CHR() value of the number of the LRC-byte //
   cChar := CHR( Byte2Num(cLRC) )
ENDIF
RETURN cChar


FUNCTION Num2Byte( nNumber )
*--------------------------------------------------------------------------
*
* Create a character repr. of the byte of a specific number.
*
* nNumber        the number to process.
* cBin           The char-repr. of the byte.
*
* Num2Byte( 189 ) -> "10111101"
*
* Byte2Num
*
*--------------------------------------------------------------------------
LOCAL nBit, cBin := "", cBit
IF nNumber<=255
   FOR nBit := 7 TO 0 STEP -1
      IF nNumber-(2^nBit)>=0
         nNumber -= (2^nBit)
         cBit    := "1"
      ELSE
         cBit    := "0"
      ENDIF
      cBin += cBit
   NEXT i
ENDIF
RETURN cBin


FUNCTION Byte2Num( cBin )
*--------------------------------------------------------------------------
*
* Calculate the numeric value of a character repr. of a byte.
*
* cBin           The char-repr. of the byte.
* nNumber        the number to process.
*
* Byte2Num( "10111101" ) -> 189
*
* Num2Byte
*
*--------------------------------------------------------------------------
LOCAL nNumber := 0, nBit
IF !EMPTY(cBin) .AND. LEN(cBin)==8
   FOR nBit := 7 TO 0 STEP -1
      IF SUBSTR(cBin, 8-nBit, 1)=="1"
         nNumber += (2^nBit)
      ENDIF
   NEXT i
ENDIF
RETURN nNumber
[end LRC.prg]


